char cmd[101];
int cmdlen;
char pinModes[22];
byte errNum; //Error flag for getNum function =0/1
byte posNum; //Next position after readed number
byte drvControl=1;

int Servo0Pos,Servo1Pos;
int Servo0Target,Servo1Target;
int Servo0Time,Servo1Time;
int Servo0Timer,Servo1Timer;

int i; //for internal loops only, may be overwriten in any function

void error_cmd(int error_id=0)
{
  Serial.print("ERR: Wrong cmd=[");
  for(i=0; i<cmdlen; i++) Serial.print(cmd[i]);
  Serial.print("] error=");
  Serial.println(error_id,DEC);
};

int calculateTimerForServoMove(int pos0, int pos1, int max_speed){
  if(pos0==0 || pos1==0) return 0;
  long int diff=pos0-pos1;
  if(diff<0) diff=-diff;
  if(max_speed==0){ return 0; };
  diff=1000*diff/max_speed;
  return diff;
};

int getServoPos(int servo_num)
{
  if(servo_num==0){
    if(Servo0Timer==0) return Servo0Pos;
    long int x=Servo0Target-Servo0Pos;
    x=x*Servo0Time;
    x=x/Servo0Timer;
    return Servo0Pos+x;
  };
  if(servo_num==1){
    if(Servo1Timer==0) return Servo1Pos;
    long int x=Servo1Target-Servo1Pos;
    x=x*Servo1Time;
    x=x/Servo1Timer;
    return Servo1Pos+x;
  };
};

void exec_servo_cmd()
{
  int st=0;
  int x;
  int servo=2;
  int servoPos0=1,servoPos1=1;
  int servoSpd0=0,servoSpd1=0;
  int timer=0;
  while(st<cmdlen){
    if(cmd[st]<=' ') st++;
    if(cmd[st]=='#'){
      st++;
      if(cmd[st]!='0' && cmd[st]!='1'){ error_cmd(20); return; };
      servo=cmd[st]-'0';
      if(servo==0){ servoPos0=0; servoSpd0=0; };
      if(servo==1){ servoPos1=0; servoSpd1=0; };
      st++;
    };
    if(cmd[st]=='P'){
      st++;
      x=getNum(st);
      st=posNum;
      if(errNum>0){ error_cmd(30+errNum); return; };
      if(servo==2){ error_cmd(21); return; };
      if(x<500 || x>2500){ error_cmd(22); return; };
      if(servo==0){ servoPos0=x; };
      if(servo==1){ servoPos1=x; };
    };
    if(cmd[st]=='S'){
      st++;
      x=getNum(st);
      st=posNum;
      if(errNum>0){ error_cmd(31+errNum); return; };
      if(servo==2){ error_cmd(23); return; };
      if(x<1 || x>30000){ error_cmd(24); return; };
      if(servo==0){ servoSpd0=x; };
      if(servo==1){ servoSpd1=x; };
    };
    if(cmd[st]=='T'){
      st++;
      x=getNum(st);
      st=posNum;
      if(errNum>0){ error_cmd(32+errNum); return; };
      if(x<0 || x>30000){ error_cmd(25); return; };
      timer=x;
    };
  };
  if(servo==2){ error_cmd(26); return; };
  if(servoPos0==0){ set_pwm1(0,0); Servo0Pos=0; Servo0Timer=0; };
  if(servoPos1==0){ set_pwm1(1,0); Servo1Pos=0; Servo1Timer=0; };
  if(servoSpd0>0 || servoSpd1>0){
    int timer0=calculateTimerForServoMove(servoPos0,Servo0Pos,servoSpd0);
    int timer1=calculateTimerForServoMove(servoPos0,Servo0Pos,servoSpd0);
    if(timer<timer0)timer=timer0;
    if(timer<timer1)timer=timer1;
  };
  if(timer==0){
    if(servoPos0>1){ set_pwm1(0,servoPos0); Servo0Pos=servoPos0; Servo0Timer=0; };
    if(servoPos1>1){ set_pwm1(1,servoPos1); Servo1Pos=servoPos1; Servo1Timer=0; };
  }else{
    if(servoPos0>1 && servoPos0!=Servo0Pos){ Servo0Pos=getServoPos(0); Servo0Timer=timer; Servo0Time=0; Servo0Target=servoPos0; };
    if(servoPos1>1 && servoPos1!=Servo1Pos){ Servo1Pos=getServoPos(1); Servo1Timer=timer; Servo1Time=0; Servo1Target=servoPos1; };
  };
};

int getNum(int startPos)
{
  char sign='+';
  int res=0;
  int sp;
  //Serial.println("StRdNm");
  sp=startPos;
  if(cmd[sp]=='-' || cmd[sp]=='+') sign=cmd[sp++];
  if( cmd[sp]<'0' || cmd[sp]>'9'){ errNum=1; return 0; };
  while( cmd[sp]>='0' && cmd[sp]<='9' && sp<101) res=res*10+(cmd[sp++]-'0');
  posNum=sp;
  errNum=0;
  //Serial.println("FnRdNm");
  if(sign=='-') return -res;
  return res;
};

void print_pin_id(int pin){
  if(pin>=0 && pin<=13){
    Serial.print("D");
    Serial.print(pin,DEC);
  } else if(pin>=14 && pin<=21){
    Serial.print("A");
    Serial.print((pin-14),DEC);
  }else{
    Serial.print("?PinId?");
  };
};

void exec_pin_cmd()
{
  int st=1;
  char pinCmd;
  int pin;
  
  if(cmd[1]=='i' && cmd[2]=='n'){ st=3; }; //ignore 'in' in Pin
  pinCmd=cmd[st];
  st++;
  if(pinCmd=='M'){
    //PinMode
    if(cmd[st]=='o' && cmd[st+1]=='d' && cmd[st+2]=='e'){ st+=3; }; //ignore 'ode' in PinMode
  }else if(pinCmd=='S' || pinCmd=='G'){
    //PinSet
    if(cmd[st]=='e' && cmd[st+1]=='t'){ st+=2; }; //ignore 'et' in PinSet/PinGet
  }else{
    error_cmd(1);
    return;
  };
  if(cmd[st]!='A' && cmd[st]!='D' ){ error_cmd(2); return; };

  pin=getNum(st+1); if(cmd[st]=='A') pin+=14;
  if( errNum ){ error_cmd(3); return; };
  st=posNum;
  if( pin<0 || pin>21){ Serial.println("ERR: Wrong pin id, allowed only D2..D13, A0..A7"); return; };
  if( drvControl==1 && pin>=7 && pin<=10 ){ Serial.println("ERR: Wrong pin id, D7..D10 reserved by DrvControl=1"); return; };
  if(pinCmd=='G'){
    if(pinModes[pin]=='A'){
      i=analogRead(pin-14);
      print_pin_id(pin);
      Serial.print("=");
      Serial.print(i,DEC);
      Serial.println("");
      return;
    };
    i=digitalRead(pin);
    print_pin_id(pin);
    Serial.print("=");
    Serial.print(i,DEC);
    Serial.println("");
    return;
  };
  
  if(cmd[st]!='='){ error_cmd(4); return; }; //for PinMode & PinSet commands must be = after pin id
  st++;
  if(pinCmd=='M'){
    if((cmd[st]=='I' || cmd[st]=='O') && (pin<2 || pin>19)){ Serial.println("ERR: PinMode 'In' or 'Out' available only for pins D2..D13, A0..A5"); return; };
    if(cmd[st]=='A' && (pin<14 || pin>21)){ Serial.println("ERR: PinMode 'Adc' available only for pins A0..A7"); return; };
    Serial.print("PinMode");
    print_pin_id(pin);
    Serial.print("=");
    pinModes[pin]=cmd[st];
    if(pin==9 || pin==10) set_pwm1(pin-9,0);
    if(cmd[st]=='A'){
      if(pin<20) pinMode(pin,INPUT);
      Serial.println("A");
    }else if(cmd[st]=='I'){
      pinMode(pin,INPUT);
      Serial.println("I");
    }else if(cmd[st]=='O'){
      pinMode(pin,OUTPUT);
      Serial.println("O");
    };
    return;
  };
  
  if(pinCmd=='S'){
    if(pinModes[pin]=='A'){ Serial.println("ERR: PinSet not allowed on pins in Adc mode"); return; };
    if(cmd[st]!='0' && cmd[st]!='1'){ Serial.println("ERR: PinSet value must be 0 or 1 only"); return; };
    if(cmd[st]=='0') digitalWrite(pin,LOW);
    if(cmd[st]=='1') digitalWrite(pin,HIGH);
    print_pin_id(pin);
    Serial.print("=");
    Serial.println(cmd[st]);
    return;
  };
};

void exec_drv_cmd()
{
  int spdL,spdR,speedL,speedR;
  int st=1;
  if(cmd[1]=='r' && cmd[2]=='v'){ st=3; }; //ignore 'rv' in Drv
  if(cmd[st]=='L'){
    if(drvControl==0){ Serial.println("ERR: DrvControl=0"); return; };
    st++;
    if(cmd[st]=='R'){ st++; }; //ignore 'R' in DrvLR
    if(cmd[st]!='='){ error_cmd(8); return; }; //for DrvLR commands must be = after command
    speedL=getNum(st+1);
    if(errNum){ error_cmd(9); return; };
    if(cmd[posNum]!=','){ error_cmd(10); return; }; //for DrvLR commands must be , after first parameter
    speedR=getNum(posNum+1);
    if(errNum){ error_cmd(10); return; };
    if(speedL<-100 || speedL>100 || speedR<-100 || speedR>100){ Serial.println("ERR: In DrvLR command parameters must be in range -100..+100"); return; };
    Serial.print("DrvLR=");
    Serial.print(speedL,DEC);
    Serial.print(',');
    Serial.println(speedR,DEC);
    if(speedL<0){ digitalWrite(7,HIGH); }else digitalWrite(7,LOW);
    if(speedR<0){ digitalWrite(8,HIGH); }else digitalWrite(8,LOW);
    spdL=speedL; if(speedL<0) spdL=-speedL;
    spdR=speedR; if(speedR<0) spdR=-speedR;
    int LPWM,RPWM;
    if(spdL==100){ LPWM=0xFF; }else{ LPWM=spdL+spdL+(spdL>>1); };
    if(spdR==100){ RPWM=0xFF; }else{ RPWM=spdR+spdR+(spdR>>1); };
    set_pwm0(LPWM,RPWM);
  }else{
    error_cmd(6);
  };
};

void exec_help_cmd(){
  Serial.println("Help :)");
};

void execute_cmd()
{
  if(cmdlen==0) return;
  for(i=cmdlen; i<101; i++) cmd[i]=10;
  //Serial.print("Cmd=[");
  //for(i=0; i<cmdlen; i++) Serial.print(cmd[i]);
  //*999Serial.println("]");
  /*
  if( cmd[0]=='?'){ exec_help_cmd(); Serial.println("H-Done"); return; };
  if( cmd[0]=='#' ){ exec_servo_cmd(); Serial.println("S-Done"); return; };
  if( cmd[0]=='P' ){ exec_pin_cmd(); Serial.println("P-Done"); return; };
  if( cmd[0]=='D' ){ exec_drv_cmd(); Serial.println("D-Done"); return; };
  */
  
  if( cmd[0]=='?'){ exec_help_cmd(); return; };
  if( cmd[0]=='#' ){ exec_servo_cmd(); return; };
  if( cmd[0]=='P' ){ exec_pin_cmd(); return; };
  if( cmd[0]=='D' ){ exec_drv_cmd(); return; };
  
  
  error_cmd(7);
}

void set_pwm0(int pwmA, int pwmB)
{
  if(pwmA==0x00 || pwmA==0xFF){
    TCCR0A=(TCCR0A & 0x3F);
    if(pwmA==0xFF){ digitalWrite(6,1); };
    if(pwmA==0x00){ digitalWrite(6,0); };
  }else if(pwmA>0){
    TCCR0A=(TCCR0A & 0x3F) | 0x80;
    OCR0A=pwmA;
  };
  
  if(pwmB==0x00 || pwmB==0xFF){
    TCCR0B=(TCCR0A & 0xCF);
    if(pwmB==0xFF){ digitalWrite(5,1); };
    if(pwmB==0x00){ digitalWrite(5,0); };
  }else if(pwmB>0){
    TCCR0A=(TCCR0A & 0xCF) | 0x20;
    OCR0B=pwmB;
  };
}

void set_pwm1(int channel, int pwm) //channel 0=A, 1=B, pwm = 500..2500ms for servo, pwm=0 - disconnect
{
  if(channel==0){
    if(pwm==0){
      TCCR1A = (TCCR1A & 0x3F); // Normal port operation mode for OC1A
    }else{
      DDRB |= (1<<PB1);
      TCCR1A = (TCCR1A & 0x3F) | (1<<COM1A1); // non-inverting mode for OC1A
      OCR1A=pwm<<1;
    };
  }else if(channel==1){
    if(pwm==0){
      TCCR1A = (TCCR1A & 0xAF); // Normal port operation mode for OC1A
    }else{
      DDRB |= (1<<PB2);
      TCCR1A = (TCCR1A & 0xAF) | (1<<COM1B1); // non-inverting mode for OC1A
      OCR1B=pwm<<1;
    };
  };
  /*
  Serial.print("set_pwm1 channel=");
  Serial.print(channel,DEC);
  Serial.print(" pwm=");
  Serial.print(pwm,DEC);
  Serial.println("");
  */
}

void setup_timer0()
{
  //Freq 16'000'000 Hz
  //Need Freq=5-10KHz
  //Prescaler 1/8
  //Result Freq = 16'000'000/256/8=7.8KHz
  TCCR0A=0x03; //Both OC0A,OC0B set to disconnected normal mode, Timer0 mode - Fast PWM
  TCCR0B=0x0A; //Timer0 mode - Fast PWM, Prescaler 1/8
  TIMSK0=0x00;
  TCNT0=0;
  OCR0A=0;
  OCR0B=0;
  
  DDRD |= 0x60;
}

void setup_timer1()
{
  //Freq = 16'000'000 Hz
  //Need Freq=50Hz
  //Prescaler 1/8
  //ICR = Freq/NeedFreq*Prescaler=16'000'000/50*1/8=40'000

  // Mode 14, Prescaler 8
  TCCR1A = (TCCR1A & 0xFA) | (1<<WGM11);
  TCCR1B = (TCCR1B & 0xF0) | (1<<WGM13) | (1<<WGM12) | (1<<CS11);
  
  ICR1 = 40000;
}

void setup_timer2() //100Hz timer
{
  TCCR2A=0x00; //Normal mode
  TCCR2B=0x07; //Prescaler 1/1024
  OCR2A=0x9C; // 16'000'000Hz/1024prescaler/100Hz=156 counter value
  TIMSK2=0x02; //Timer 2 output compare match A interrupt enabled
  TCNT2=0;
}

ISR(SIG_OUTPUT_COMPARE2A)
{
  TCNT2=0;
  //process
  if(Servo0Timer>0){
    Servo0Time+=10;
    if(Servo0Time<Servo0Timer){
      set_pwm1(0,getServoPos(0));
    }else{
      Servo0Timer=0;
      Servo0Pos=Servo0Target;
      set_pwm1(0,Servo0Pos);
    };
  };
  if(Servo1Timer>0){
    Servo1Time+=10;
    if(Servo1Time<Servo1Timer){
      set_pwm1(1,getServoPos(1));
    }else{
      Servo1Timer=1;
      Servo1Pos=Servo1Target;
      set_pwm1(1,Servo1Pos);
    };
  };
}

void setup()
{
   Serial.begin(19200);
   cmdlen=0;
   for(i=2; i<=13; i++) pinModes[i]='I';
   for(i=14; i<=21; i++) pinModes[i]='A';
   setup_timer0();
   setup_timer1();
   setup_timer2();
   set_pwm0(0,0);
   sei();
}

void loop()
{
  while (1)
  while (Serial.available() > 0) {
    char data=Serial.read();
    if( data==10 ){ execute_cmd(); cmdlen=0; }
    else if( data<33 ) continue;
    else{ cmd[cmdlen++]=data; };
    if( cmdlen==100 ){
      Serial.println("ERR: Command to long, 100bytes max.");
      cmdlen=0;
    };
  };
}
